"use strict";
// *****************************************************************************
// Copyright (C) 2021 TypeFox and others.
//
// This program and the accompanying materials are made available under the
// terms of the Eclipse Public License v. 2.0 which is available at
// http://www.eclipse.org/legal/epl-2.0.
//
// This Source Code may also be made available under the following Secondary
// Licenses when the conditions for such availability set forth in the Eclipse
// Public License v. 2.0 are satisfied: GNU General Public License, version 2
// with the GNU Classpath Exception which is available at
// https://www.gnu.org/software/classpath/license.html.
//
// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
// *****************************************************************************
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalizationBackendContribution = void 0;
const inversify_1 = require("inversify");
const promise_util_1 = require("../../common/promise-util");
const localization_contribution_1 = require("./localization-contribution");
const localization_provider_1 = require("./localization-provider");
let LocalizationBackendContribution = class LocalizationBackendContribution {
    constructor() {
        this.initialized = new promise_util_1.Deferred();
    }
    async initialize() {
        await this.localizationRegistry.initialize();
        this.initialized.resolve();
    }
    waitForInitialization() {
        return this.initialized.promise;
    }
    configure(app) {
        app.get('/i18n/:locale', async (req, res) => {
            await this.waitForInitialization();
            let locale = req.params.locale;
            locale = this.localizationProvider.getAvailableLanguages().some(e => e.languageId === locale) ? locale : 'en';
            this.localizationProvider.setCurrentLanguage(locale);
            res.send(this.localizationProvider.loadLocalization(locale));
        });
    }
};
__decorate([
    (0, inversify_1.inject)(localization_contribution_1.LocalizationRegistry),
    __metadata("design:type", localization_contribution_1.LocalizationRegistry)
], LocalizationBackendContribution.prototype, "localizationRegistry", void 0);
__decorate([
    (0, inversify_1.inject)(localization_provider_1.LocalizationProvider),
    __metadata("design:type", localization_provider_1.LocalizationProvider)
], LocalizationBackendContribution.prototype, "localizationProvider", void 0);
LocalizationBackendContribution = __decorate([
    (0, inversify_1.injectable)()
], LocalizationBackendContribution);
exports.LocalizationBackendContribution = LocalizationBackendContribution;
//# sourceMappingURL=localization-backend-contribution.js.map